﻿// イメージビュア 16x16版
// 公開された16x16ドット イメージデータを一覧・拡大表示します
// ※別途イメージデータファイル(ImgData1-3.int)が必要です
// STARTボタンを押すと終了します

// イメージデータについて
//http://www.picosoft.co.jp/ORANGE-light/image1616.html
//Copyright (c) 2019 Picosoft Co., Ltd.
//Made by Kaori Hanatsumi

//下記"30"を、ImgData1.intを保存したEEPROMスロット番号に書き換えてください
FSLOT = 30-1

' 色定数の定義
WHITE = rgb(255, 255, 255)
DARKGRAY = rgb(64, 64, 64)
ORANGE = rgb(255, 163, 0)
LBLUE = rgb(0, 140, 255)
DARKBLUE = rgb(0, 0, 64)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_LEFT = 7
BUTTON_RIGHT = 6
BUTTON_START = 9

' ワークエリア定義
OFFSET = getmptr()
mdataw 0
SBUF = getmptr()
mptr SBUF+8
BUF = getmptr()

i2ci 1,400

cls 2
for i=0 to 149
	img_get(i,WHITE,DARKBLUE)
next

gprint 0,232,"Push START to Exit",ORANGE,0


' メインループ
x=1:y=0:img_zoom(1)
while in(BUTTON_START)
	pre_i = y*10 + x
	x = x + trig_right() - trig_left()
	y = y + trig_down() - trig_up()
	x = x + ((x<0) - (x>9))*10
	y = y + ((y<0) - (y>14))*15
	i = y*10 + x
	if i <> pre_i then img_get(pre_i,WHITE,DARKBLUE):img_zoom(i)
wend
end

' イメージデータを読んで表示
def img_get(n,col1,col2) {
	index_read(n,SBUF)
	if mpeek(SBUF) = -1 then return 1
	img_read(n,BUF)
	gput16c(10 + n%10,n/10,BUF,col1,col2)
	return 0
}

' 読み込み済みのイメージを拡大表示
def img_zoom(i) {
	var cx
	var cy
	var d
	var c
	if img_get(i,DARKGRAY,ORANGE) then rect 0,11,160,160,0,1 : return
	rect 0,0,64,8,0,1
	gp_str(0,0,SBUF,LBLUE,8)
	cy=15
	while cy>=0
		d=mpeekw(BUF/2+cy)
		cx=15
		while cx>=0
			if d&1 then c=WHITE else c=0
			rect cx*10+1,cy*10+11,8,8,c,1
			d=d>>1:cx=cx-1
		wend
		cy=cy-1
	wend
}

' メモリー配列にある文字列を表示
def gp_str(x,y,a,col,n) {
	while n
		gprint x*8,y*8,chr$(mpeek(a)),col
		a=a+1:x=x+1:n=n-1
	wend
}

' イメージ名をEEPROMから読み込む
def index_read(id,ptr) {
	mpokew OFFSET/2,FSLOT*2048 + 4928 + id*8
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,8,1)
}

' イメージデータをEEPROMから読み込む
def img_read(id,ptr) {
	mpokew OFFSET/2,FSLOT*2048 + id*32
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,32,1)
}

' 16ドット単位で16x16イメージデータを表示
def gput16c(x,y,adr,col1,col2) {
	sprnew(0,x*16,y*16,16,16,adr,col1,col2)
	sprdsp 0,1
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_left() {
	var ret
	ret = pre_l
	pre_l = in(BUTTON_LEFT)
	return ret & !pre_l
}
def trig_right() {
	var ret
	ret = pre_r
	pre_r = in(BUTTON_RIGHT)
	return ret & !pre_r
}