// イメージビュア 8x8版
// 公開された8x8ドット イメージデータを一覧・拡大表示します
// ※別途イメージデータファイル(Img8x8.int)が必要です
// STARTボタンを押すと終了します

// イメージデータについて
// http://www.picosoft.co.jp/ORANGE-light/SystemCharacters.pdf
// Copyright (c) 2019 Picosoft Co., Ltd.
// Made by Kaori Hanatsumi

//下記"29"を、Img8x8.intを保存したEEPROMスロット番号に書き換えてください
FSLOT = 29-1

' 色定数の定義
WHITE = rgb(255, 255, 255)
DARKGRAY = rgb(64, 64, 64)
ORANGE = rgb(255, 163, 0)
LBLUE = rgb(0, 140, 255)
DARKBLUE = rgb(0, 0, 64)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_LEFT = 7
BUTTON_RIGHT = 6
BUTTON_START = 9

' ワークエリア定義
OFFSET = getmptr()
mptr OFFSET + 2
BUF = getmptr()

i2ci 1,400

cls 2
for i=0 to 129
	img_get(i,WHITE,DARKBLUE)
next

gprint 0,232,"Push START to Exit",ORANGE,0

' メインループ
x=0:y=0:img_zoom(0)
while in(BUTTON_START)
	pre_i = y*10 + x
	x = x + trig_right() - trig_left()
	y = y + trig_down() - trig_up()
	x = x + ((x<0) - (x>9))*10
	y = y + ((y<0) - (y>12))*13
	i = y*10 + x
	if i <> pre_i then img_get(pre_i,WHITE,DARKBLUE):img_zoom(i)
wend
end

' イメージデータを読んで表示
def img_get(n,col1,col2) {
	img_read(n,BUF)
	gput8c(10 + n%10,n/10,BUF,col1,col2)
}

' 読み込み済みのイメージを拡大表示
def img_zoom(i) {
	var cx
	var cy
	var d
	var c
	img_get(i,DARKGRAY,ORANGE)
	cy=7
	while cy>=0
		d=mpeek(BUF+cy)
		cx=7
		while cx>=0
			if d&1 then c=WHITE else c=0
			rect cx*10+1,cy*10+11,8,8,c,1
			d=d>>1:cx=cx-1
		wend
		cy=cy-1
	wend
}

' イメージデータをEEPROMから読み込む
def img_read(id,ptr) {
	mpokew OFFSET/2,FSLOT*2048 + 32 + id*8
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,8,1)
}

' 16ドット単位で8x8イメージデータを表示
def gput8c(x,y,adr,col1,col2) {
	sprnew(0,x*16,y*16,8,8,adr,col1,col2)
	sprdsp 0,1
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_left() {
	var ret
	ret = pre_l
	pre_l = in(BUTTON_LEFT)
	return ret & !pre_l
}
def trig_right() {
	var ret
	ret = pre_r
	pre_r = in(BUTTON_RIGHT)
	return ret & !pre_r
}