// EEPROMのファイルリストを表示する
// 上下ボタンでファイルスロットを選択すると
// 内容を16x16のドットイメージとして表示します

cls 2

' 色定数の定義
WHITE = rgb(255, 255, 255)
GREEN = rgb(0, 255, 0)
ORANGE = rgb(255, 163, 0)
DARKBLUE = rgb(0, 0, 64)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_B = 10
BUTTON_START = 9

' 変数初期化
gp_x = 0
gp_y = 0
gp_color = WHITE

Ymax=19
Smax=31-Ymax
y=0
s=0
Cmin=3
Cmax=Ymax-3

' スプライトパターン定義
CS01 = getmptr()
mdata 0xc0,0xf0,0xfc,0xff,0xfc,0xf0,0xc0,0

' ワークエリア定義
OFFSET = getmptr()
mptr OFFSET + 2
BUF = getmptr()
mptr BUF + 256

i2ci 1,400

gprint 176,232,"Push START to Exit",ORANGE,0
window(0,0,20,20)
view_list(s)

' スプライト作成・表示
sprnew(1,8,8+y*8,8,8,CS01,WHITE,0)
sprdsp 1,1
window(22,0,16,16)
view_image(s+y)

' メインループ
while in(BUTTON_START)
	d = up_check() | down_check() 
	if d&1 then view_list(s)
	if d&2 then sprmov 1,8,8+y*8
	if d then view_image(s+y)
	if trig_b() then eeprom_dump(s+y)
wend
end

' カーソル上移動チェック スクロール付
def up_check() {
	if !trig_up() then return 0
	if (y=Cmin) and (s>0) then s=s-1:return 1
	y=y-1:if y<0 then s=Smax:y=Ymax:return 3
	return 2
}

' カーソル下移動チェック スクロール付
def down_check() {
	if !trig_down() then return 0
	if (y=Cmax) and (s<Smax) then s=s+1:return 1
	y=y+1:if y>Ymax then s=0:y=0:return 3
	return 2
}

' ファイルリストを表示
def view_list(n) {
	var i
	rect 16,8,152,160,0,1
	for i=0 to Ymax
		eeprom_read((i+n)*2048,BUF,32)
		gp_lc(2,i+1,GREEN):gp_num(i+n+1,2)
		if mpeekw(BUF/2) != 0x7069 then goto skip
		gp_lc(5,i+1,WHITE):gp_mem(BUF+4,8)
	//	gp_num(mpeekw((BUF+14)/2),4)
	//	gp_hex(mpeekw((BUF+18)/2),4)
		gp_lc(13,i+1,ORANGE):gp_num(mpeekw((BUF+22)/2),4)
	//	gp_hex(mpeekw((BUF+26)/2),4)
		gp_lc(17,i+1,WHITE):gp_num(mpeekw((BUF+30)/2),4)
	skip:
	next
}

' EEPROMから読み込んで16x16イメージとして表示
def view_image(n) {
	var x
	for x=0 to 7
		eeprom_read(n*2048 + x*256 ,BUF ,256)
		sprnew(0,184+x*16,8,16,128,BUF,WHITE,DARKBLUE)
		sprdsp 0,1
	next
}

def eeprom_dump(n) {
	var i
	var len
	var ptr
	var r

	eeprom_read(n*2048,BUF,32)
	len = 32+ mpeekw(BUF/2 + 11) + mpeekw(BUF/2 + 15)
// len = 2048

	ptr=0
	while len
		if len<32 then r=len else r=32
		eeprom_read(n*2048 + ptr,BUF,r)
		for i=0 to r-1
			print hex$(mpeek(BUF+i));
			if (i&15)=15 then print
		next
		len = len - r
		ptr = ptr + r
	wend
	print "$"
}
def eeprom_read(adr,ptr,len) {
	var e
	mpokew OFFSET/2,adr
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,len,1)
}

' LCD文字表示用 便利関数
def gp_lc(x,y,col) {
	gp_x = x
	gp_y = y
	gp_color = col
}

' メモリー配列内の文字列を n文字分 表示
def gp_mem(a,n) {
	while n
		gprint gp_x*8,gp_y*8,chr$(mpeek(a)),gp_color
		a = a + 1:gp_x = gp_x + 1:n = n - 1
	wend
}

' 数値を n桁分 表示
def gp_num(d,n) {
	var i
	var first
	var c
	i = n
	first = 1
	while i
		if d || first then gprint (gp_x+i-1)*8,gp_y*8,d%10,gp_color
		i = i - 1
		d = d / 10
		first = 0
	wend
	gp_x = gp_x + n
}

' 数値を16進数で n桁 表示
def gp_hex(d,n) {
	var i
	var c
	i = n
	while i
		c = d & 15
		c = c+48 + 7*(c>9)
		gprint (gp_x+i-1)*8,gp_y*8,chr$(c),gp_color
		d = d / 16
		i = i - 1
	wend
	gp_x = gp_x + n
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def window(x,y,w,h) {
	var i
	gp(x,y,0x9C)
	gp(x,y+h+1,0x9E)
	for i=1 to w
		gp(x+i,y,0x97)
		gp(x+i,y+h+1,0x97)
	next
	gp(x+w+1,y,0x9D)
	gp(x+w+1,y+h+1,0x9F)

	for i=1 to h
		gp(x,y+i,0x96)
		gp(x+w+1,y+i,0x96)
	next

	rect (x+1)*8,(y+1)*8,w*8,h*8,0,1
}
def gp(x,y,c) {
	gprint x*8,y*8,chr$(c),0xFFFF,0
}

def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_b() {
	var ret
	ret = pre_b
	pre_b = in(BUTTON_B)
	return ret & !pre_b
}